@@ -8,6 +8,7 @@ import os
 import ssl
 import datetime
 import irods.password_obfuscation as obf
+from irods import MAX_NAME_LEN
 from ast import literal_eval as safe_eval
 
 
@@ -15,7 +16,13 @@ from irods.message import (
     iRODSMessage, StartupPack, AuthResponse, AuthChallenge, AuthPluginOut,
     OpenedDataObjRequest, FileSeekResponse, StringStringMap, VersionResponse,
     PluginAuthMessage, ClientServerNegotiation, Error, GetTempPasswordOut)
-from irods.exception import get_exception_by_code, NetworkException
+from irods.exception import (get_exception_by_code, NetworkException, nominal_code)
+from irods.message import (PamAuthRequest, PamAuthRequestOut)
+
+
+ALLOW_PAM_LONG_TOKENS = True      # True to fix [#279]
+
+
 from irods import (
     MAX_PASSWORD_LENGTH, RESPONSE_LEN,
     AUTH_SCHEME_KEY, AUTH_USER_KEY, AUTH_PWD_KEY, AUTH_TTL_KEY,
@@ -91,37 +98,31 @@ class Connection(object):
             self.release(True)
             raise NetworkException("Unable to send message")
 
-    def recv(self):
+    def recv(self, into_buffer = None
+                 , return_message = ()
+                 , acceptable_errors = ()):
+        acceptable_codes = set(nominal_code(e) for e in acceptable_errors)
         try:
-            msg = iRODSMessage.recv(self.socket)
+            if into_buffer is None:
+                msg = iRODSMessage.recv(self.socket)
+            else:
+                msg = iRODSMessage.recv_into(self.socket, into_buffer)
         except socket.error:
             logger.error("Could not receive server response")
             self.release(True)
             raise NetworkException("Could not receive server response")
+        if isinstance(return_message,list): return_message[:] = [msg]
         if msg.int_info < 0:
             try:
                 err_msg = iRODSMessage(msg=msg.error).get_main_message(Error).RErrMsg_PI[0].msg
             except TypeError:
-                raise get_exception_by_code(msg.int_info)
-            raise get_exception_by_code(msg.int_info, err_msg)
+                err_msg = None
+            if nominal_code(msg.int_info) not in acceptable_codes:
+                raise get_exception_by_code(msg.int_info, err_msg)
         return msg
 
-    def recv_into(self, buffer):
-        try:
-            msg = iRODSMessage.recv_into(self.socket, buffer)
-        except socket.error:
-            logger.error("Could not receive server response")
-            self.release(True)
-            raise NetworkException("Could not receive server response")
-
-        if msg.int_info < 0:
-            try:
-                err_msg = iRODSMessage(msg=msg.error).get_main_message(Error).RErrMsg_PI[0].msg
-            except TypeError:
-                raise get_exception_by_code(msg.int_info)
-            raise get_exception_by_code(msg.int_info, err_msg)
-
-        return msg
+    def recv_into(self, buffer, **options):
+        return self.recv( into_buffer = buffer, **options )
 
     def __enter__(self):
         return self
@@ -161,7 +162,13 @@ class Connection(object):
             context = self.account.ssl_context
         except AttributeError:
             CA_file = getattr(self.account, 'ssl_ca_certificate_file', None)
+            verify_server_mode = getattr(self.account,'ssl_verify_server', 'hostname')
+            if verify_server_mode == 'none':
+                CA_file = None
             context = ssl.create_default_context(ssl.Purpose.SERVER_AUTH, cafile=CA_file)
+            if CA_file is None:
+                context.check_hostname = False
+                context.verify_mode = 0  # VERIFY_NONE
 
         # Wrap socket with context
         wrapped_socket = context.wrap_socket(self.socket, server_hostname=host)
@@ -402,9 +409,11 @@ class Connection(object):
 
     def _login_pam(self):
 
+        time_to_live_in_seconds = 60
+
         ctx_user = '%s=%s' % (AUTH_USER_KEY, self.account.client_user)
         ctx_pwd = '%s=%s' % (AUTH_PWD_KEY, self.account.password)
-        ctx_ttl = '%s=%s' % (AUTH_TTL_KEY, "60")
+        ctx_ttl = '%s=%s' % (AUTH_TTL_KEY, str(time_to_live_in_seconds))
 
         ctx = ";".join([ctx_user, ctx_pwd, ctx_ttl])
 
@@ -412,23 +421,34 @@ class Connection(object):
             if getattr(self,'DISALLOWING_PAM_PLAINTEXT',True):
                 raise PlainTextPAMPasswordError
 
-        message_body = PluginAuthMessage(
-            auth_scheme_=PAM_AUTH_SCHEME,
-            context_=ctx
-        )
+        Pam_Long_Tokens = (ALLOW_PAM_LONG_TOKENS and (len(ctx) >= MAX_NAME_LEN))
+
+        if Pam_Long_Tokens:
+
+            message_body = PamAuthRequest(
+                pamUser=self.account.client_user,
+                pamPassword=self.account.password,
+                timeToLive=time_to_live_in_seconds)
+        else:
+
+            message_body = PluginAuthMessage(
+                auth_scheme_ = PAM_AUTH_SCHEME,
+                context_ = ctx)
 
         auth_req = iRODSMessage(
             msg_type='RODS_API_REQ',
             msg=message_body,
-            # int_info=725
-            int_info=1201
+            int_info=(725 if Pam_Long_Tokens else 1201)
         )
 
         self.send(auth_req)
         # Getting the new password
         output_message = self.recv()
 
-        auth_out = output_message.get_main_message(AuthPluginOut)
+        Pam_Response_Class = (PamAuthRequestOut if Pam_Long_Tokens
+                         else AuthPluginOut)
+
+        auth_out = output_message.get_main_message( Pam_Response_Class )
 
         self.disconnect()
         self._connect()
